import boto3
import json

from lib.application_configs import ApplicationConfigs

session = boto3.Session(
    aws_access_key_id=ApplicationConfigs.aws_access_key_id,
    aws_secret_access_key=ApplicationConfigs.aws_secret_key)

# ToDo: Figure out a much better way of doing this.
# LocalStack wants endpoint_url, while prod doesnt :(
if ApplicationConfigs.local_dev_env:
    s3  = session.resource('s3', region_name=ApplicationConfigs.region_name, endpoint_url=ApplicationConfigs.endpoint_url)
    sqs = session.client('sqs', region_name=ApplicationConfigs.region_name, endpoint_url=ApplicationConfigs.endpoint_url)
else:
    s3  = session.resource('s3', region_name=ApplicationConfigs.region_name)
    sqs = session.client('sqs', region_name=ApplicationConfigs.region_name)

def get_key_from_message(body):
    return body['Records'][0]['s3']['object']['key']


def get_bucket_from_message(body):
    return body['Records'][0]['s3']['bucket']['name']


def get_object(key, bucket):
    return s3.Object(bucket_name=bucket, key=key).get()['Body'].read()


def file_stream_upload(buffer, name, bucket):
    return s3.Bucket(bucket).upload_fileobj(buffer, name)


def receive_message(queue, message_num=1, wait_time=1):
    return sqs.receive_message(QueueUrl=queue,
                               MaxNumberOfMessages=message_num,
                               WaitTimeSeconds=wait_time)


def delete_message(queue, receipt):
    return sqs.delete_message(QueueUrl=queue, ReceiptHandle=receipt)
